package com.sourcetohtml;

import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

/**
 * Manages configuration of the application
 * Use setProperty method to override the configuration at run time
 * Please note this has to be done before first invocation generation!!
 * @author jcelak
 */
public class Config {
	private static Config c = new Config();
	
	private Properties p;
	
	private Config(){
		try {
			p = new Properties();
			InputStream in = getClass().getResourceAsStream("/sourcetohtml-app.properties");
			if (in == null)
				in = getClass().getResourceAsStream("/sourcetohtml.properties");
			p.load(in);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	public static Config getInstance()
	{
		return c;
	}
	
	public String getProperty(String key, String defaultValue)
	{
		String result = p.getProperty(key);
		if (result == null)
			return defaultValue;
		return result;
	}

	public boolean getBooleanProperty(String key, boolean defaultValue)
	{
		String result = p.getProperty(key);
		if (result == null)
			return defaultValue;
		return Boolean.valueOf(result);
	}
	
	/**
	 * overrides property
	 * @param key key of the property
	 * @param value value of the property
	 */
	public void setProperty(String key, String value)
	{
		p.setProperty(key, value);
	}
}
