package com.sourcetohtml;

import java.awt.Color;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.imageio.ImageIO;
import javax.imageio.ImageWriter;
import javax.imageio.stream.ImageOutputStream;
import javax.swing.text.TabExpander;

import org.gjt.sp.jedit.Buffer;
import org.gjt.sp.jedit.Mode;
import org.gjt.sp.jedit.jEdit;
import org.gjt.sp.jedit.syntax.Chunk;
import org.gjt.sp.jedit.syntax.DisplayTokenHandler;
import org.gjt.sp.jedit.syntax.ModeProvider;
import org.gjt.sp.jedit.syntax.SyntaxStyle;
import org.gjt.sp.jedit.syntax.Token;
import org.gjt.sp.jedit.syntax.TokenMarker;
import org.gjt.sp.util.SyntaxUtilities;

/**
 * Uses JEdit to determinate type of file and to parse the file into tokens
 * It than iterates through the tokens and generates HTML
 * and a thumbnail image if requested 
 * @author jcelak
 */
public class JEditGenerator extends FileDetailGenerator
{
    private static final Logger log = Logger.getLogger("TextToHtml");
    
    private static final int thumbnailWidth = 100;
    private static final int thumbnailHeight = 100;
    
    private SyntaxStyle[] syntaxStyles = null;
    
    public static void main(String arg[]) throws Exception
    { 
		JEditGenerator tth = new JEditGenerator();               
		tth.write(new File("c:\\CheckoutBeanValidatorTest.java"),new File("c:/temp/a.html"), new File("c:/temp/s.gif"));
        //tth.write(new File("c:\\temp\\test\\archive\\jirka\\archive\\_ARCH_file merge.zip\\file merge\\cobal\\layout.jsp"),new File("c:/temp/b.html"));        
        //tth.write(new File("c:\\temp\\hchfinance\\menu_files\\ddlevelsmenu-topbar.css"),new File("c:/temp/c.html"));
        //tth.write(new File("c:\\temp\\hchfinance\\menu_files\\ddlevelsmenu.js"),new File("c:/temp/d.html"));
    }
    
    
    public JEditGenerator()
    {
        init();
    }
    
    @Override
	public String getHTML(File input, File root) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		write(input, new OutputStreamWriter(baos), null);
		return baos.toString();
	}


    /**
     * Initialize Jedit by loading modes and initialize syntax styles
     */
	private void init()
    {
        jEdit.initSystemProperties();
        jEdit.reloadModes();
        SyntaxUtilities.propertyManager = jEdit.getPropertyManager();
        String defaultFont = jEdit.getProperty("view.font");
        int defaultFontSize = jEdit.getIntegerProperty("view.fontsize",12);
        syntaxStyles = SyntaxUtilities.loadStyles(defaultFont,defaultFontSize);
        syntaxStyles[0] = syntaxStyles[1];
    }
        
    private String readFirstLine(File f) throws Exception   
    {
        BufferedReader in = null;
        try {
            in = new BufferedReader(new FileReader(f));
            return in.readLine();
        } finally {
            if (in != null)
                in.close();
        }
    }
    
    public synchronized void write(File source, File destination, File thumbnailDestination)
    {
    	try {
			write(source,new FileWriter(destination),thumbnailDestination);
		} catch (IOException e) {
			e.printStackTrace();
		}
    }
    
    public void write(File source, Writer out, File thumbnailDestination)
    {
        BufferedImage thumbImage = null;
        try {
        	Graphics2D graphics2D = null;
        	if (thumbnailDestination != null)
        	{
	        	thumbImage = new BufferedImage(thumbnailWidth, thumbnailHeight,
	                    BufferedImage.TYPE_INT_ARGB);
	        	graphics2D = thumbImage.createGraphics();
	            graphics2D.setColor(new Color(255,255,255));
	            graphics2D.fillRect(0, 0, thumbnailWidth, thumbnailHeight);
        	}
            ModeProvider modeprovider = ModeProvider.instance;
            Mode m = modeprovider.getModeForFile(source.getAbsolutePath(), 
                    readFirstLine(source));
            Buffer b = jEdit.openTemporary(null, source.getParentFile().getAbsolutePath(), source.getName(), false);
            out = new BufferedWriter(out);
            TokenMarker tm = m.getTokenMarker();        
            b.setTokenMarker(tm);
            out.write("<table cellpadding=\"0\" cellspacing=\"0\" id=\"sourceTable\"><tbody>");            
            out.write("<tr>");
            if (Config.getInstance().getBooleanProperty("com.sourcetohtml.JEditGenerator.generateLineNumbers", true))
            {
	            out.write("<td valign=\"top\"><div id='lineNumber'><pre>");
	            for (int i=0;i<b.getLineCount();i++)
	            {
	                out.write("<a href=\"#L"+(i+1)+"\" name=\"L"+(i+1)+"\">"+(i+1)+"</a>\n");
	            }
	            out.write("</pre></div></td>");
            }
            out.write("<td style=\"padding-left:0.2cm\" td valign=\"top\"><div id=\"CodeDiv\"><pre>\n");
            int x = 1;
            int y = 1;
            for (int i=0;i<b.getLineCount();i++)
            {                       
                DisplayTokenHandler tokenHandler = new DisplayTokenHandler();
                tokenHandler.init(syntaxStyles,null,new TabExpander()
                                        {
                                            public float nextTabStop(float x, int tabOffset)
                                            {
                                                return 0;
                                            }
                                            
                                        },new ArrayList<Chunk>(),0.0f);
                b.markTokens(i, tokenHandler);
                //System.out.print("<tr><td class=\"lineNumber\">"+(i+1)+"</td><td style=\"padding-left:0.2cm\">");
                Token t = tokenHandler.getTokens(); 
                boolean emptyLine = true;
                int maxHeight = 3;
                while (t != null)
                {
                    Chunk c = (Chunk)t;
                    if (c.str != null)
                    {                    
                    	if (y < thumbnailHeight && x < thumbnailWidth && thumbnailDestination != null)
                    	{
	                    	Font cf = c.style.getFont();                    	
	                    	Font f = new Font("arial",cf.getStyle(),6);
	                    	graphics2D.setFont(f);
	                    	graphics2D.setColor(c.style.getForegroundColor());
	                    	FontMetrics fm = graphics2D.getFontMetrics();
	                        int width = fm.stringWidth(c.str);
	                        int height = fm.getAscent();
	                    	graphics2D.drawString(c.str, x, y + height);
	                    	x = x + width + 2;
	                    	maxHeight = Math.max(height, maxHeight);
                    	}
                        out.write("<span class=\""+getCSSClass(c.style)+"\">" + 
                                escapeSpecialCharacters(c.str) + //.replaceAll(" ", "&nbsp;&nbsp;") + 
                                "</span>");
                        emptyLine = false;
                    } else
                        out.write("\t");
                        //System.out.println("&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;");
                    t = t.next;
                }  
                x = 1;
                y = y + maxHeight + 1;
                if (emptyLine)
                	out.write("<span class=\""+getCSSClass(syntaxStyles[0])+"\">&nbsp;</span>");
                out.write("\n");
            }   
            out.write("</pre></div></td></tr>\n");
            out.write("</tbody></table>\n");
        } catch (Exception e)
        {
            log.log(Level.SEVERE, "Unable to generate html version of file " + source.getAbsolutePath(),e);
        } finally
        {
            if (out != null)
            {
                try {
                    out.close();
                } catch (IOException e) {}
            }
            if (thumbnailDestination != null)
            	saveImage(thumbImage,thumbnailDestination);
        }
    }
    
    private void saveImage(BufferedImage thumbImage, File destination)
    {
    	ImageWriter writer = null;
        Iterator iter = ImageIO.getImageWritersByFormatName("gif");
        if (iter.hasNext())
        {
            writer = (ImageWriter) iter.next();
        }
        ImageOutputStream ios = null;
		try {
			ios = ImageIO.createImageOutputStream(destination);
			writer.setOutput(ios);
  
			writer.write(thumbImage);
		} catch (IOException e) {
			log.log(Level.SEVERE, "Unable store text thumbnail " + destination.getAbsolutePath(),e);
		} finally
		{
			if (ios != null)
			{
				try {
					ios.flush();
					ios.close();
				}
				catch (Exception e)
				{
					log.log(Level.SEVERE, "Unable close streams ",e);
				}
			}
	        writer.dispose();	        
		}
    }
    
    private String escapeSpecialCharacters(String input)
    {
        return input.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
    }
    
    private String getCSSClass(SyntaxStyle ss)
    {
    	for (int i= 0;i<syntaxStyles.length;i++)
    	{
    		if (syntaxStyles[i] == ss)
    			return "s" + i;
    	}
    	return "s0";
    }
    
}
